//
//  Copyright(C) 2017 OPTOELECTRONICS CO., LTD. All rights reserved.
//
/*******************************************************************************/

#ifndef SRC_FTP_H_
#define SRC_FTP_H_

/** FTP_Command command **/
#define FTP_CMD_USERNAME	1
#define FTP_CMD_PASSWORD	2
#define FTP_CMD_CHPATH		3
#define FTP_CMD_TYPESET		4
#define FTP_CMD_PUT			6
#define FTP_CMD_GET			8
#define FTP_CMD_DEL			12
#define FTP_CMD_RENAME		92
#define FTP_CMD_QUIT		9

/** PUT/GET progress message **/
#define FTP_MSG_PUT_PROGRESS_LENGTH		90
#define FTP_MSG_GET_PROGRESS_LENGTH		91

/** FTP error **/
#define FTP_ERROR			100
#define FTP_ERROR_CMD		101
#define FTP_ERROR_FILENAME	102
#define FTP_ERROR_FILEOPEN	103
#define FTP_ERROR_FILEWRITE	104
#define FTP_ERROR_FILESEEK	105

/** SetFtpMode u8Mode **/
#define FTP_ACTIVE			0
#define FTP_PASV			1

/** CreateRemoteFilename filenameType **/
#define FTP_NAME_NOTIME_NOID	0
#define FTP_NAME_TIME_NOID		1
#define FTP_NAME_NOTIME_ID		2
#define FTP_NAME_TIME_ID		3
#define FTP_NAME_FREE			4

typedef void (*tpfResponseCb) (unsigned char command, char *msg);

typedef void (*tpfProgressCb) (unsigned char command, unsigned int value);

/** FTP_Connect
 * Connect to FTP server
 * @param pcHost
 * 		FTP server IP address or host name. Max : 64 string.
 * @param port
 * 		FTP connection port.
 * @param mode
 * 		FTP_PASV : set passive mode (general)
 * 		FTP_ACTIVE : set active mode
 * @param responseFunction
 * 		Register response message callback. If NULL to do nothing.
 * @return
 */
int FTP_Connect(char *pcHost, unsigned short port, unsigned char mode, tpfResponseCb responseFunction);

/** FTP_ConnectEx
 * Connect to FTP server
 * @param pcHost
 * 		FTP server IP address or host name. Max : 64 string.
 * @param port
 * 		FTP connection port.
 * @param mode
 * 		FTP_PASV : set passive mode (general)
 * 		FTP_ACTIVE : set active mode
 * @param responseFunction
 * 		Register response message callback. If NULL to do nothing.
 * @param progressFunction
 * 		Register progress message callback. If NULL to do nothing.
 * @return
 */
int FTP_ConnectEx(char *pcHost, unsigned short port, unsigned char mode, tpfResponseCb responseFunction, tpfProgressCb progressFunction);

/** FTP_Command
 * Send to FTP command.
 * @param command
 * 		FTP_CMD_USERNAME
 * 		FTP_CMD_PASSWORD
 * 		FTP_CMD_CHPATH
 * 		FTP_CMD_TYPESET
 * 		FTP_CMD_PUT
 * 		FTP_CMD_GET
 * 		FTP_CMD_DEL
 * 		FTP_CMD_RENAME
 * 		FTP_CMD_QUIT
 *
 * @param arg1(char *)
 * 		@command=FTP_CMD_USERNAME
 * 			username
 * 		@command=FTP_CMD_PASSWORD
 * 			password
 * 		@command=FTP_CMD_CHPATH
 * 			remote change path
 * 		@command=FTP_CMD_TYPESET
 * 			set type
 * 		@command=FTP_CMD_PUT
 * 			local filename to put
 * 		@command=FTP_CMD_GET
 * 			local filename to get
 * 		@command=FTP_CMD_DEL
 * 			remote filename to delete
 * 		@command=FTP_CMD_RENAME
 * 			source filename to rename
 * @param arg2(char *)
 * 		@command=FTP_CMD_PUT
 * 			remote filename to put
 * 		@command=FTP_CMD_GET
 * 			remote filename to get
 * 		@command=FTP_CMD_RENAME
 * 			destination filename to rename
 *
 * @return M2M_SUCCESS if success, FTP_ERROR/FTP_ERROR_CMD if error.
 */
int FTP_Command(unsigned char command, char *param1, char *param2);

/** FTP_Finish
 * Finish FTP client.
 */
void FTP_Finish(void);

/** FTP_CreateRemoteFilename
 * Create remote filename additional timestamp/terminal ID.
 * (Optional)
 * @param filenameType
 * 		FTP_NAME_NOTIME_NOID	: Same local filename.
 * 		FTP_NAME_TIME_NOID		: Add timestamp.
 * 		FTP_NAME_NOTIME_ID		: Add terminal ID.
 * 		FTP_NAME_TIME_ID		: Add timestamp & terminal ID.
 * @param filename
 * 		local file name
 * @param remoteName
 * 		remote file name
 * @param tempName
 * 		temporary file name
 * @return
 */
int FTP_CreateRemoteFilename(int filenameType, const char *filename, char *remoteName, char *tempName);

/** FTP_Log
 * Switch FTP log.
 * (Optional)
 * @param ena
 * 		disable : FTP log to disable.
 * 		enable : FTP log to enable.
 */
void FTP_Log(int ena);
#endif /* SRC_FTP_H_ */
